@@ -6,7 +6,7 @@ from typing import List, Optional
 
 from dataclasses_jsonschema import JsonSchemaMixin
 
-from arcor2.data import common
+from arcor2.data import common, execution
 
 
 def wo_suffix(name: str) -> str:
@@ -43,6 +43,19 @@ Project / scene
 """
 
 
+@dataclass
+class OpenSceneData(JsonSchemaMixin):
+
+    scene: common.Scene
+
+
+@dataclass
+class OpenScene(Event):
+
+    data: Optional[OpenSceneData] = None
+    event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
+
+
 @dataclass
 class SceneChanged(Event):
 
@@ -56,6 +69,12 @@ class SceneSaved(Event):
     event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
 
 
+@dataclass
+class SceneClosed(Event):
+
+    event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
+
+
 @dataclass
 class ProjectChanged(Event):
 
@@ -63,12 +82,32 @@ class ProjectChanged(Event):
     event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
 
 
+@dataclass
+class OpenProjectData(JsonSchemaMixin):
+
+    scene: common.Scene
+    project: common.Project
+
+
+@dataclass
+class OpenProject(Event):
+
+    data: Optional[OpenProjectData] = None
+    event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
+
+
 @dataclass
 class ProjectSaved(Event):
 
     event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
 
 
+@dataclass
+class ProjectClosed(Event):
+
+    event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
+
+
 @dataclass
 class ActionPointChanged(Event):
 
@@ -147,14 +186,23 @@ class CurrentActionEvent(Event):
 
 
 @dataclass
-class ProjectStateEvent(Event):
+class PackageStateEvent(Event):
 
-    data: common.ProjectState = field(default_factory=common.ProjectState)
+    data: common.PackageState = field(default_factory=common.PackageState)
     event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
 
 # ----------------------------------------------------------------------------------------------------------------------
 
 
+@dataclass
+class PackageInfoEvent(Event):
+
+    data: Optional[execution.PackageInfo] = None
+    event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
+
+
+# ----------------------------------------------------------------------------------------------------------------------
+
 @dataclass
 class ActionStateEvent(Event):
 
@@ -196,6 +244,15 @@ class ActionResultEvent(Event):
     event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
 
 
+@dataclass
+class ActionCancelledEvent(Event):
+    """
+    Used to notify that the action execution was cancelled.
+    """
+
+    event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
+
+
 """
 ------------------------------------------------------------------------------------------------------------------------
 Objects
@@ -208,3 +265,45 @@ class ObjectTypesChangedEvent(Event):
 
     data: List[str] = field(default_factory=list)  # changed object types
     event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
+
+
+"""
+------------------------------------------------------------------------------------------------------------------------
+Robots
+------------------------------------------------------------------------------------------------------------------------
+"""
+
+
+@dataclass
+class RobotJointsData(JsonSchemaMixin):
+
+    robot_id: str
+    joints: List[common.Joint]
+
+
+@dataclass
+class RobotJointsEvent(Event):
+
+    data: Optional[RobotJointsData] = None
+    event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
+
+
+@dataclass
+class EefPose(JsonSchemaMixin):
+
+    end_effector_id: str
+    pose: common.Pose
+
+
+@dataclass
+class RobotEefData(JsonSchemaMixin):
+
+    robot_id: str
+    end_effectors: List[EefPose] = field(default_factory=list)
+
+
+@dataclass
+class RobotEefEvent(Event):
+
+    data: Optional[RobotEefData] = None
+    event: str = field(default=wo_suffix(__qualname__), init=False)  # type: ignore  # noqa: F821
