@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 
-from os import path
+import os
 from typing import Dict, Union, TypeVar, List, Optional, Any, Type
 import importlib
 import json
@@ -11,8 +11,9 @@ from dataclasses_jsonschema import JsonSchemaValidationError, JsonSchemaMixin
 
 from arcor2.object_types_utils import built_in_types_names
 from arcor2.data.common import Project, Scene, CurrentAction, Pose, Orientation
-from arcor2.data.object_type import ObjectModel, Models
-from arcor2.data.events import CurrentActionEvent
+from arcor2.data.execution import PackageInfo
+from arcor2.data.object_type import ObjectModel, Models, Box, Sphere, Cylinder, Mesh
+from arcor2.data.events import CurrentActionEvent, PackageInfoEvent
 from arcor2.exceptions import ResourcesException, Arcor2Exception
 import arcor2.object_types
 from arcor2.object_types import Generic
@@ -66,6 +67,13 @@ class IntResources:
 
         built_in = built_in_types_names()
 
+        if self.robot_service:
+            self.robot_service.clear_collisions()
+
+        package_info_event = PackageInfoEvent()
+        # TODO read package id from package.json
+        package_info_event.data = PackageInfo(os.path.basename(os.getcwd()), self.scene, self.project)
+
         for scene_obj in self.scene.objects:
 
             if scene_obj.type in built_in:
@@ -89,15 +97,35 @@ class IntResources:
                 for srv_type in obj_meta.needs_services:
                     args.append(self.services[srv_type])
 
-                inst = cls(*args, scene_obj.id, scene_obj.pose, models[scene_obj.type])
+                # TODO type hints
+                inst = cls(*args, scene_obj.id, scene_obj.name, scene_obj.pose, models[scene_obj.type])
             else:
-                inst = cls(scene_obj.id, scene_obj.pose, models[scene_obj.type])
+                # TODO type hints
+                inst = cls(scene_obj.id, scene_obj.name, scene_obj.pose, models[scene_obj.type])
 
             self.objects[scene_obj.id] = inst
 
             if self.robot_service:
                 self.robot_service.add_collision(inst)
 
+        if self.robot_service:
+
+            for model in models.values():
+
+                if not model:
+                    continue
+
+                if isinstance(model, Box):
+                    package_info_event.data.collision_models.boxes.append(model)
+                elif isinstance(model, Sphere):
+                    package_info_event.data.collision_models.spheres.append(model)
+                elif isinstance(model, Cylinder):
+                    package_info_event.data.collision_models.cylinders.append(model)
+                elif isinstance(model, Mesh):
+                    package_info_event.data.collision_models.meshes.append(model)
+
+        print_event(package_info_event)
+
         self.all_instances: Dict[str, Union[Generic, Service]] = dict(**self.objects, **self.services)
 
     def make_all_poses_absolute(self) -> None:
@@ -174,7 +202,7 @@ class ResourcesBase(IntResources):
 
         try:
 
-            with open(path.join("data", file_name + ".json")) as scene_file:
+            with open(os.path.join("data", file_name + ".json")) as scene_file:
 
                 data_dict = json.loads(scene_file.read())
                 data_dict = convert_keys(data_dict, camel_case_to_snake_case)
