@@ -17,6 +17,8 @@ class RestException(Arcor2Exception):
 
 TIMEOUT = (1.0, 20.0)  # connect, read
 
+HEADERS = {'accept': 'application/json', 'content-type': 'application/json'}
+
 T = TypeVar('T', bound=JsonSchemaMixin)
 S = TypeVar('S', str, int, float, bool)
 
@@ -71,7 +73,7 @@ def handle_response(resp: requests.Response) -> None:
 
         try:
             raise RestException(resp_body["message"], str(e)) from e
-        except KeyError:
+        except (KeyError, TypeError):  # TypeError is for case when resp_body is just string
             raise RestException(str(resp_body), str(e)) from e
 
 
@@ -82,7 +84,10 @@ def _send(url: str, op: Callable, data: OptionalData = None,
         if isinstance(data, list):
             d = []
             for dd in data:
-                d.append(convert_keys(dd.to_dict(), snake_case_to_camel_case))
+                if isinstance(dd, str):
+                    d.append(dd)  # type: ignore
+                else:
+                    d.append(convert_keys(dd.to_dict(), snake_case_to_camel_case))  # type: ignore
         else:
             d = convert_keys(data.to_dict(), snake_case_to_camel_case)  # type: ignore
     else:
@@ -94,7 +99,7 @@ def _send(url: str, op: Callable, data: OptionalData = None,
         params = {}
 
     try:
-        resp = op(url, data=json.dumps(d), timeout=TIMEOUT, headers={'Content-Type': 'application/json'},
+        resp = op(url, data=json.dumps(d), timeout=TIMEOUT, headers=HEADERS,
                   params=params)
     except requests.exceptions.RequestException as e:
         raise RestException("Catastrophic system error.", str(e)) from e
@@ -160,7 +165,7 @@ def put_returning_list(url: str, data: OptionalData = None,
 def delete(url: str):
 
     try:
-        resp = SESSION.delete(url, timeout=TIMEOUT)
+        resp = SESSION.delete(url, timeout=TIMEOUT, headers=HEADERS)
     except requests.exceptions.RequestException as e:
         raise RestException("Catastrophic system error.", str(e)) from e
 
@@ -190,7 +195,8 @@ def _get_response(url: str, body: Optional[JsonSchemaMixin] = None, params: Para
         params = {}
 
     try:
-        resp = SESSION.get(url, timeout=TIMEOUT, data=body_dict, params=params)
+        resp = SESSION.get(url, timeout=TIMEOUT, data=body_dict, params=params, allow_redirects=True,
+                           headers=HEADERS)
     except requests.exceptions.RequestException as e:
         raise RestException("Catastrophic system error.", str(e)) from e
 
@@ -245,9 +251,10 @@ def get_list(url: str, data_cls: Type[T], body: Optional[JsonSchemaMixin] = None
     return ret
 
 
-def get_list_primitive(url: str, desired_type: Type[S]) -> List[S]:
+def get_list_primitive(url: str, desired_type: Type[S], body: Optional[JsonSchemaMixin] = None,
+                       params: ParamsDict = None) -> List[S]:
 
-    data = get_data(url)
+    data = get_data(url, body, params)
 
     ret: List[S] = []
 
@@ -269,15 +276,9 @@ def get(url: str, data_cls: Type[T], body: Optional[JsonSchemaMixin] = None, par
         raise RestException("Invalid data.", str(e)) from e
 
 
-def download(url: str, path: str) -> None:
+def download(url: str, path: str, body: Optional[JsonSchemaMixin] = None, params: ParamsDict = None) -> None:
     # TODO check content type
 
-    try:
-        r = SESSION.get(url, allow_redirects=True)
-    except requests.exceptions.RequestException as e:
-        raise RestException("Download of file failed.", str(e)) from e
-
-    handle_response(r)
-
+    r = _get_response(url, body, params)
     with open(path, 'wb') as file:
         file.write(r.content)
