@@ -3,15 +3,16 @@
 
 
 import asyncio
-from typing import Union, List, Dict, Any, Optional, Set
+from typing import Dict, Any, Set
 import copy
 from contextlib import asynccontextmanager
+import inspect
+
+from websockets.server import WebSocketServerProtocol as WsClient
 
 from arcor2 import object_types_utils as otu, helpers as hlp
 from arcor2.data import rpc, events, common, object_type
 from arcor2 import aio_persistent_storage as storage
-from arcor2.object_types import Generic
-from arcor2.services import Service
 from arcor2.exceptions import Arcor2Exception
 from arcor2.parameter_plugins import PARAM_PLUGINS
 from arcor2.parameter_plugins.base import ParameterPluginException
@@ -21,7 +22,7 @@ from arcor2.server.decorators import scene_needed, project_needed, no_project
 from arcor2.server import objects_services_actions as osa, notifications as notif, globals as glob
 from arcor2.server.robot import get_end_effector_pose, get_robot_joints
 from arcor2.server.project import project_problems, open_project, project_names
-from arcor2.server.scene import open_scene, clear_scene
+from arcor2.server.scene import open_scene, clear_scene, get_instance
 
 
 def find_object_action(action: common.Action) -> object_type.ObjectAction:
@@ -78,13 +79,58 @@ def unique_name(name: str, existing_names: Set[str]) -> None:
 
 @scene_needed
 @project_needed
-async def execute_action_cb(req: rpc.project.ExecuteActionRequest) -> \
-        Union[rpc.project.ExecuteActionResponse, hlp.RPC_RETURN_TYPES]:
+async def cancel_action_cb(req: rpc.project.CancelActionRequest, ui: WsClient) -> None:
+
+    assert glob.PROJECT
+
+    if not glob.RUNNING_ACTION:
+        raise Arcor2Exception("No action is running.")
+
+    action = glob.PROJECT.action(glob.RUNNING_ACTION)
+    obj_id, action_name = action.parse_type()
+    obj = get_instance(obj_id)
+
+    for act in glob.ACTIONS[obj.__class__.__name__]:
+        if act.name != action_name:
+            continue
+
+        if not act.meta.cancellable:
+            raise Arcor2Exception("Action is not cancellable.")
+
+    try:
+        action_method = getattr(obj, action_name)
+        cancel_method = getattr(obj, obj.CANCEL_MAPPING[action_method.__name__])
+    except AttributeError as e:
+        raise Arcor2Exception("Internal error.") from e
+
+    cancel_params: Dict[str, Any] = {}
+
+    cancel_sig = inspect.signature(cancel_method)
+
+    assert glob.RUNNING_ACTION_PARAMS is not None
+
+    for param_name, param in cancel_sig.parameters.items():
+        try:
+            cancel_params[param_name] = glob.RUNNING_ACTION_PARAMS[param_name]
+        except KeyError as e:
+            raise Arcor2Exception("Cancel method parameters should be subset of action parameters.") from e
+
+    await hlp.run_in_executor(cancel_method, *cancel_params.values())
+
+    asyncio.ensure_future(notif.broadcast_event(events.ActionCancelledEvent()))
+    glob.RUNNING_ACTION = None
+    glob.RUNNING_ACTION_PARAMS = None
+
+
+@scene_needed
+@project_needed
+async def execute_action_cb(req: rpc.project.ExecuteActionRequest, ui: WsClient) -> None:
 
     assert glob.SCENE and glob.PROJECT
 
     if glob.RUNNING_ACTION:
-        return False, f"Action {glob.RUNNING_ACTION} is being executed. Only one action can be executed at a time."
+        raise Arcor2Exception(f"Action {glob.RUNNING_ACTION} is being executed. "
+                              f"Only one action can be executed at a time.")
 
     action = glob.PROJECT.action(req.args.action_id)
 
@@ -96,23 +142,17 @@ async def execute_action_cb(req: rpc.project.ExecuteActionRequest) -> \
                                                                          glob.PROJECT, action.id, param.id)
         except ParameterPluginException as e:
             await glob.logger.error(e)
-            return False, f"Failed to get value for parameter {param.id}."
+            raise Arcor2Exception(f"Failed to get value for parameter {param.id}.")
 
     obj_id, action_name = action.parse_type()
 
-    obj: Optional[Union[Generic, Service]] = None
-
-    if obj_id in glob.SCENE_OBJECT_INSTANCES:
-        obj = glob.SCENE_OBJECT_INSTANCES[obj_id]
-    elif obj_id in glob.SERVICES_INSTANCES:
-        obj = glob.SERVICES_INSTANCES[obj_id]
-    else:
-        return False, "Internal error: project not in sync with scene."
+    obj = get_instance(obj_id)
 
     if not hasattr(obj, action_name):
-        return False, "Internal error: object does not have the requested method."
+        raise Arcor2Exception("Internal error: object does not have the requested method.")
 
     glob.RUNNING_ACTION = action.id
+    glob.RUNNING_ACTION_PARAMS = params
 
     await glob.logger.debug(f"Running action {action.name} ({type(obj)}/{action_name}), params: {params}.")
 
@@ -121,54 +161,53 @@ async def execute_action_cb(req: rpc.project.ExecuteActionRequest) -> \
     return None
 
 
-async def list_projects_cb(req: rpc.project.ListProjectsRequest) -> \
-        Union[rpc.project.ListProjectsResponse, hlp.RPC_RETURN_TYPES]:
+async def project_info(project_id: str, scenes_lock: asyncio.Lock, scenes: Dict[str, common.Scene]) -> \
+        rpc.project.ListProjectsResponseData:
 
-    data: List[rpc.project.ListProjectsResponseData] = []
+    project = await storage.get_project(project_id)
+
+    pd = rpc.project.ListProjectsResponseData(id=project.id, desc=project.desc, name=project.name,
+                                              scene_id=project.scene_id)
+
+    try:
+        async with scenes_lock:
+            if project.scene_id not in scenes:
+                scenes[project.scene_id] = await storage.get_scene(project.scene_id)
+    except storage.PersistentStorageException:
+        pd.problems.append("Scene does not exist.")
+        return pd
+
+    pd.problems = project_problems(scenes[project.scene_id], project)
+    pd.valid = not pd.problems
+
+    if not pd.valid:
+        return pd
+
+    try:
+        program_src(project, scenes[project.scene_id], otu.built_in_types_names())
+        pd.executable = True
+    except SourceException as e:
+        pd.problems.append(e.message)
+
+    return pd
+
+
+async def list_projects_cb(req: rpc.project.ListProjectsRequest, ui: WsClient) -> rpc.project.ListProjectsResponse:
 
     projects = await storage.get_projects()
 
+    scenes_lock = asyncio.Lock()
     scenes: Dict[str, common.Scene] = {}
 
-    # TODO do this in parallel?
-    for project_iddesc in projects.items:
-
-        try:
-            project = await storage.get_project(project_iddesc.id)
-        except Arcor2Exception as e:
-            await glob.logger.warning(f"Ignoring project {project_iddesc.id} due to error: {e}")
-            continue
-
-        pd = rpc.project.ListProjectsResponseData(id=project.id, desc=project.desc, name=project.name,
-                                                  scene_id=project.scene_id)
-        data.append(pd)
-
-        if project.scene_id not in scenes:
-            try:
-                scenes[project.scene_id] = await storage.get_scene(project.scene_id)
-            except storage.PersistentStorageException:
-                pd.problems.append("Scene does not exist.")
-                continue
-
-        pd.problems = project_problems(scenes[project.scene_id], project)
-        pd.valid = not pd.problems
-
-        if not pd.valid:
-            continue
-
-        try:
-            program_src(project, scenes[project.scene_id], otu.built_in_types_names())
-            pd.executable = True
-        except SourceException as e:
-            pd.problems.append(e.message)
-
-    return rpc.project.ListProjectsResponse(data=data)
+    resp = rpc.project.ListProjectsResponse()
+    tasks = [project_info(project_iddesc.id, scenes_lock, scenes) for project_iddesc in projects.items]
+    resp.data = await asyncio.gather(*tasks)
+    return resp
 
 
 @scene_needed
 @project_needed
-async def add_action_point_joints_cb(req: rpc.project.AddActionPointJointsRequest) -> \
-        Union[rpc.project.AddActionPointJointsResponse, hlp.RPC_RETURN_TYPES]:
+async def add_action_point_joints_cb(req: rpc.project.AddActionPointJointsRequest, ui: WsClient) -> None:
 
     assert glob.SCENE and glob.PROJECT
 
@@ -176,6 +215,9 @@ async def add_action_point_joints_cb(req: rpc.project.AddActionPointJointsReques
 
     unique_name(req.args.name, ap.joints_names())
 
+    if req.dry_run:
+        return None
+
     new_joints = await get_robot_joints(req.args.robot_id)
 
     prj = common.ProjectRobotJoints(common.uid(), req.args.name, req.args.robot_id, new_joints, True)
@@ -187,8 +229,7 @@ async def add_action_point_joints_cb(req: rpc.project.AddActionPointJointsReques
 
 @scene_needed
 @project_needed
-async def update_action_point_joints_cb(req: rpc.project.UpdateActionPointJointsRequest) -> \
-        Union[rpc.project.UpdateActionPointJointsResponse, hlp.RPC_RETURN_TYPES]:
+async def update_action_point_joints_cb(req: rpc.project.UpdateActionPointJointsRequest, ui: WsClient) -> None:
 
     assert glob.SCENE and glob.PROJECT
 
@@ -205,8 +246,7 @@ async def update_action_point_joints_cb(req: rpc.project.UpdateActionPointJoints
 
 @scene_needed
 @project_needed
-async def remove_action_point_joints_cb(req: rpc.project.RemoveActionPointJointsRequest) -> \
-        Union[rpc.project.RemoveActionPointJointsResponse, hlp.RPC_RETURN_TYPES]:
+async def remove_action_point_joints_cb(req: rpc.project.RemoveActionPointJointsRequest, ui: WsClient) -> None:
     """
     Removes joints from action point.
     :param req:
@@ -220,7 +260,7 @@ async def remove_action_point_joints_cb(req: rpc.project.RemoveActionPointJoints
     for act in glob.PROJECT.actions():
         for param in act.parameters:
             if PARAM_PLUGINS[param.type].uses_robot_joints(glob.PROJECT, act.id, param.id, req.args.joints_id):
-                return False, f"Joints used in action {act.name} (parameter {param.id})."
+                raise Arcor2Exception(f"Joints used in action {act.name} (parameter {param.id}).")
 
     joints_to_be_removed = ap.joints(req.args.joints_id)
     ap.robot_joints = [joints for joints in ap.robot_joints if joints.id != req.args.joints_id]
@@ -233,8 +273,7 @@ async def remove_action_point_joints_cb(req: rpc.project.RemoveActionPointJoints
 
 @scene_needed
 @project_needed
-async def rename_action_point_cb(req: rpc.project.RenameActionPointRequest) -> \
-        Union[rpc.project.RenameActionPointResponse, hlp.RPC_RETURN_TYPES]:
+async def rename_action_point_cb(req: rpc.project.RenameActionPointRequest, ui: WsClient) -> None:
 
     assert glob.SCENE and glob.PROJECT
 
@@ -244,9 +283,13 @@ async def rename_action_point_cb(req: rpc.project.RenameActionPointRequest) -> \
         return None
 
     if not hlp.is_valid_identifier(req.args.new_name):
-        return False, "Name has to be valid Python identifier."
+        raise Arcor2Exception("Name has to be valid Python identifier.")
 
     unique_name(req.args.new_name, glob.PROJECT.action_points_names)
+
+    if req.dry_run:
+        return None
+
     ap.name = req.args.new_name
 
     glob.PROJECT.update_modified()
@@ -257,8 +300,7 @@ async def rename_action_point_cb(req: rpc.project.RenameActionPointRequest) -> \
 
 @scene_needed
 @project_needed
-async def update_action_point_parent_cb(req: rpc.project.UpdateActionPointParentRequest) -> \
-        Union[rpc.project.UpdateActionPointParentResponse, hlp.RPC_RETURN_TYPES]:
+async def update_action_point_parent_cb(req: rpc.project.UpdateActionPointParentRequest, ui: WsClient) -> None:
 
     assert glob.SCENE and glob.PROJECT
 
@@ -309,8 +351,7 @@ async def update_action_point_parent_cb(req: rpc.project.UpdateActionPointParent
 
 @scene_needed
 @project_needed
-async def update_action_point_position_cb(req: rpc.project.UpdateActionPointPositionRequest) -> \
-        Union[rpc.project.UpdateActionPointPositionResponse, hlp.RPC_RETURN_TYPES]:
+async def update_action_point_position_cb(req: rpc.project.UpdateActionPointPositionRequest, ui: WsClient) -> None:
 
     assert glob.SCENE and glob.PROJECT
 
@@ -330,8 +371,7 @@ async def update_action_point_position_cb(req: rpc.project.UpdateActionPointPosi
 
 @scene_needed
 @project_needed
-async def update_action_point_using_robot_cb(req: rpc.project.UpdateActionPointUsingRobotRequest) -> \
-        Union[rpc.project.UpdateActionPointUsingRobotResponse, hlp.RPC_RETURN_TYPES]:
+async def update_action_point_using_robot_cb(req: rpc.project.UpdateActionPointUsingRobotRequest, ui: WsClient) -> None:
 
     assert glob.SCENE and glob.PROJECT
 
@@ -359,8 +399,7 @@ async def update_action_point_using_robot_cb(req: rpc.project.UpdateActionPointU
 
 @scene_needed
 @project_needed
-async def add_action_point_orientation_cb(req: rpc.project.AddActionPointOrientationRequest) -> \
-        Union[rpc.project.AddActionPointOrientationResponse, hlp.RPC_RETURN_TYPES]:
+async def add_action_point_orientation_cb(req: rpc.project.AddActionPointOrientationRequest, ui: WsClient) -> None:
     """
     Adds orientation and joints to the action point.
     :param req:
@@ -372,6 +411,10 @@ async def add_action_point_orientation_cb(req: rpc.project.AddActionPointOrienta
     ap = glob.PROJECT.action_point(req.args.action_point_id)
 
     unique_name(req.args.name, ap.orientation_names())
+
+    if req.dry_run:
+        return None
+
     orientation = common.NamedOrientation(common.uid(), req.args.name, req.args.orientation)
     ap.orientations.append(orientation)
 
@@ -383,8 +426,8 @@ async def add_action_point_orientation_cb(req: rpc.project.AddActionPointOrienta
 
 @scene_needed
 @project_needed
-async def update_action_point_orientation_cb(req: rpc.project.UpdateActionPointOrientationRequest) -> \
-        Union[rpc.project.UpdateActionPointOrientationUsingRobotResponse, hlp.RPC_RETURN_TYPES]:
+async def update_action_point_orientation_cb(req: rpc.project.UpdateActionPointOrientationRequest, ui: WsClient) -> \
+        None:
     """
     Updates orientation of the action point.
     :param req:
@@ -403,8 +446,8 @@ async def update_action_point_orientation_cb(req: rpc.project.UpdateActionPointO
 
 @scene_needed
 @project_needed
-async def add_action_point_orientation_using_robot_cb(req: rpc.project.AddActionPointOrientationUsingRobotRequest) -> \
-        Union[rpc.project.AddActionPointOrientationUsingRobotResponse, hlp.RPC_RETURN_TYPES]:
+async def add_action_point_orientation_using_robot_cb(req: rpc.project.AddActionPointOrientationUsingRobotRequest,
+                                                      ui: WsClient) -> None:
     """
     Adds orientation and joints to the action point.
     :param req:
@@ -415,6 +458,10 @@ async def add_action_point_orientation_using_robot_cb(req: rpc.project.AddAction
 
     ap = glob.PROJECT.action_point(req.args.action_point_id)
     unique_name(req.args.name, ap.orientation_names())
+
+    if req.dry_run:
+        return None
+
     new_pose = await get_end_effector_pose(req.args.robot.robot_id, req.args.robot.end_effector)
 
     if ap.parent:
@@ -433,8 +480,7 @@ async def add_action_point_orientation_using_robot_cb(req: rpc.project.AddAction
 @scene_needed
 @project_needed
 async def update_action_point_orientation_using_robot_cb(
-        req: rpc.project.UpdateActionPointOrientationUsingRobotRequest) -> \
-        Union[rpc.project.UpdateActionPointOrientationUsingRobotResponse, hlp.RPC_RETURN_TYPES]:
+        req: rpc.project.UpdateActionPointOrientationUsingRobotRequest, ui: WsClient) -> None:
     """
     Updates orientation and joint of the action point.
     :param req:
@@ -461,8 +507,8 @@ async def update_action_point_orientation_using_robot_cb(
 
 @scene_needed
 @project_needed
-async def remove_action_point_orientation_cb(req: rpc.project.RemoveActionPointOrientationRequest) -> \
-        Union[rpc.project.RemoveActionPointOrientationResponse, hlp.RPC_RETURN_TYPES]:
+async def remove_action_point_orientation_cb(req: rpc.project.RemoveActionPointOrientationRequest, ui: WsClient) -> \
+        None:
     """
     Removes orientation.
     :param req:
@@ -476,7 +522,10 @@ async def remove_action_point_orientation_cb(req: rpc.project.RemoveActionPointO
     for act in glob.PROJECT.actions():
         for param in act.parameters:
             if PARAM_PLUGINS[param.type].uses_orientation(glob.PROJECT, act.id, param.id, req.args.orientation_id):
-                return False, f"Orientation used in action {act.name} (parameter {param.id})."
+                raise Arcor2Exception(f"Orientation used in action {act.name} (parameter {param.id}).")
+
+    if req.dry_run:
+        return None
 
     ap.orientations = [ori for ori in ap.orientations if ori.id != req.args.orientation_id]
 
@@ -485,23 +534,26 @@ async def remove_action_point_orientation_cb(req: rpc.project.RemoveActionPointO
     return None
 
 
-async def open_project_cb(req: rpc.project.OpenProjectRequest) -> \
-        Union[rpc.project.OpenProjectResponse, hlp.RPC_RETURN_TYPES]:
+async def open_project_cb(req: rpc.project.OpenProjectRequest, ui: WsClient) -> None:
+
+    if glob.PACKAGE_STATE.state != common.PackageStateEnum.STOPPED:
+        raise Arcor2Exception("Can't open project while package runs.")
 
     # TODO validate using project_problems?
-    try:
-        await open_project(req.args.id)
-    except Arcor2Exception as e:
-        await glob.logger.exception(f"Failed to open project {req.args.id}.")
-        return False, e.message
+    await open_project(req.args.id)
+
+    assert glob.SCENE
+    assert glob.PROJECT
+
+    asyncio.ensure_future(notif.broadcast_event(events.OpenProject(data=events.OpenProjectData(glob.SCENE,
+                                                                                               glob.PROJECT))))
 
     return None
 
 
 @scene_needed
 @project_needed
-async def save_project_cb(req: rpc.project.SaveProjectRequest) -> \
-        Union[rpc.project.SaveProjectResponse, hlp.RPC_RETURN_TYPES]:
+async def save_project_cb(req: rpc.project.SaveProjectRequest, ui: WsClient) -> None:
 
     assert glob.SCENE and glob.PROJECT
     await storage.update_project(glob.PROJECT)
@@ -511,14 +563,19 @@ async def save_project_cb(req: rpc.project.SaveProjectRequest) -> \
 
 
 @no_project
-async def new_project_cb(req: rpc.project.NewProjectRequest) -> Union[rpc.project.NewProjectResponse,
-                                                                      hlp.RPC_RETURN_TYPES]:
+async def new_project_cb(req: rpc.project.NewProjectRequest, ui: WsClient) -> None:
+
+    if glob.PACKAGE_STATE.state != common.PackageStateEnum.STOPPED:
+        raise Arcor2Exception("Can't create project while package runs.")
 
     unique_name(req.args.name, (await project_names()))
 
+    if req.dry_run:
+        return None
+
     if glob.SCENE:
         if glob.SCENE.id != req.args.scene_id:
-            return False, "Another scene is opened."
+            raise Arcor2Exception("Another scene is opened.")
 
         if glob.SCENE.has_changes():
             await storage.update_scene(glob.SCENE)
@@ -527,45 +584,52 @@ async def new_project_cb(req: rpc.project.NewProjectRequest) -> Union[rpc.projec
     else:
 
         if req.args.scene_id not in {scene.id for scene in (await storage.get_scenes()).items}:
-            return False, "Unknown scene id."
+            raise Arcor2Exception("Unknown scene id.")
 
         await open_scene(req.args.scene_id)
 
     glob.PROJECT = common.Project(common.uid(), req.args.name, req.args.scene_id, desc=req.args.desc,
                                   has_logic=req.args.has_logic)
-    asyncio.ensure_future(notif.broadcast_event(events.ProjectChanged(events.EventType.ADD, data=glob.PROJECT)))
+
+    assert glob.SCENE
+
+    asyncio.ensure_future(notif.broadcast_event(events.OpenProject(data=events.OpenProjectData(glob.SCENE,
+                                                                                               glob.PROJECT))))
     return None
 
 
 @scene_needed
 @project_needed
-async def close_project_cb(req: rpc.project.CloseProjectRequest) -> Union[rpc.project.CloseProjectResponse,
-                                                                          hlp.RPC_RETURN_TYPES]:
+async def close_project_cb(req: rpc.project.CloseProjectRequest, ui: WsClient) -> None:
+
     assert glob.PROJECT
 
     if not req.args.force and glob.PROJECT.has_changes():
-        return False, "Project has unsaved changes."
+        raise Arcor2Exception("Project has unsaved changes.")
+
+    if req.dry_run:
+        return None
 
     glob.PROJECT = None
-    asyncio.ensure_future(notif.broadcast_event(events.ProjectChanged(events.EventType.UPDATE)))
-
     await clear_scene()
-    asyncio.ensure_future(notif.broadcast_event(events.SceneChanged(events.EventType.UPDATE)))
+    asyncio.ensure_future(notif.broadcast_event(events.ProjectClosed()))
 
     return None
 
 
 @scene_needed
 @project_needed
-async def add_action_point_cb(req: rpc.project.AddActionPointRequest) -> \
-        Union[rpc.project.AddActionPointResponse, hlp.RPC_RETURN_TYPES]:
+async def add_action_point_cb(req: rpc.project.AddActionPointRequest, ui: WsClient) -> None:
 
     assert glob.PROJECT
 
     unique_name(req.args.name, glob.PROJECT.action_points_names)
 
     if not hlp.is_valid_identifier(req.args.name):
-        return False, "Name has to be valid Python identifier."
+        raise Arcor2Exception("Name has to be valid Python identifier.")
+
+    if req.dry_run:
+        return None
 
     ap = common.ProjectActionPoint(common.uid(), req.args.name, req.args.position, req.args.parent)
     glob.PROJECT.action_points.append(ap)
@@ -577,8 +641,7 @@ async def add_action_point_cb(req: rpc.project.AddActionPointRequest) -> \
 
 @scene_needed
 @project_needed
-async def remove_action_point_cb(req: rpc.project.RemoveActionPointRequest) -> \
-        Union[rpc.project.RemoveActionPointResponse, hlp.RPC_RETURN_TYPES]:
+async def remove_action_point_cb(req: rpc.project.RemoveActionPointRequest, ui: WsClient) -> None:
 
     assert glob.PROJECT
 
@@ -589,14 +652,17 @@ async def remove_action_point_cb(req: rpc.project.RemoveActionPointRequest) -> \
 
             for joints in ap.robot_joints:
                 if PARAM_PLUGINS[param.type].uses_robot_joints(glob.PROJECT, act.id, param.id, joints.id):
-                    return False, f"Joints {joints.name} used in action {act.name} (parameter {param.id})."
+                    raise Arcor2Exception(f"Joints {joints.name} used in action {act.name} (parameter {param.id}).")
 
             for ori in ap.orientations:
                 if PARAM_PLUGINS[param.type].uses_orientation(glob.PROJECT, act.id, param.id, ori.id):
-                    return False, f"Orientation {ori.name} used in action {act.name} (parameter {param.id})."
+                    raise Arcor2Exception(f"Orientation {ori.name} used in action {act.name} (parameter {param.id}).")
 
             # TODO some hypothetical parameter type could use just bare ActionPoint (its position)
 
+    if req.dry_run:
+        return None
+
     glob.PROJECT.action_points = [acp for acp in glob.PROJECT.action_points if acp.id != req.args.id]
     glob.PROJECT.update_modified()
     asyncio.ensure_future(notif.broadcast_event(events.ActionPointChanged(events.EventType.REMOVE, data=ap.bare())))
@@ -636,8 +702,7 @@ def check_action_params(project: common.Project, action: common.Action, object_a
 
 @scene_needed
 @project_needed
-async def add_action_cb(req: rpc.project.AddActionRequest) -> \
-        Union[rpc.project.AddActionResponse, hlp.RPC_RETURN_TYPES]:
+async def add_action_cb(req: rpc.project.AddActionRequest, ui: WsClient) -> None:
 
     assert glob.PROJECT
     assert glob.SCENE
@@ -647,7 +712,7 @@ async def add_action_cb(req: rpc.project.AddActionRequest) -> \
     unique_name(req.args.name, glob.PROJECT.action_user_names())
 
     if not hlp.is_valid_identifier(req.args.name):
-        return False, "Action name has to be valid Python identifier."
+        raise Arcor2Exception("Action name has to be valid Python identifier.")
 
     new_action = common.Action(common.uid(), req.args.name, req.args.type, req.args.parameters)
 
@@ -657,6 +722,9 @@ async def add_action_cb(req: rpc.project.AddActionRequest) -> \
 
     check_action_params(updated_project, new_action, find_object_action(new_action))
 
+    if req.dry_run:
+        return None
+
     ap.actions.append(new_action)
 
     glob.PROJECT.update_modified()
@@ -666,8 +734,7 @@ async def add_action_cb(req: rpc.project.AddActionRequest) -> \
 
 @scene_needed
 @project_needed
-async def update_action_cb(req: rpc.project.UpdateActionRequest) -> Union[rpc.project.UpdateActionResponse,
-                                                                          hlp.RPC_RETURN_TYPES]:
+async def update_action_cb(req: rpc.project.UpdateActionRequest, ui: WsClient) -> None:
 
     assert glob.PROJECT
     assert glob.SCENE
@@ -679,6 +746,9 @@ async def update_action_cb(req: rpc.project.UpdateActionRequest) -> Union[rpc.pr
 
     check_action_params(updated_project, updated_action, find_object_action(updated_action))
 
+    if req.dry_run:
+        return None
+
     orig_action = glob.PROJECT.action(req.args.action_id)
     orig_action.parameters = updated_action.parameters
 
@@ -704,14 +774,17 @@ def check_action_usage(action: common.Action) -> None:
 
 @scene_needed
 @project_needed
-async def remove_action_cb(req: rpc.project.RemoveActionRequest) -> Union[rpc.project.RemoveActionResponse,
-                                                                          hlp.RPC_RETURN_TYPES]:
+async def remove_action_cb(req: rpc.project.RemoveActionRequest, ui: WsClient) -> None:
 
     assert glob.PROJECT
     assert glob.SCENE
 
     ap, action = glob.PROJECT.action_point_and_action(req.args.id)
     check_action_usage(action)
+
+    if req.dry_run:
+        return None
+
     ap.actions = [act for act in ap.actions if act.id != req.args.id]
 
     glob.PROJECT.update_modified()
@@ -721,8 +794,7 @@ async def remove_action_cb(req: rpc.project.RemoveActionRequest) -> Union[rpc.pr
 
 @scene_needed
 @project_needed
-async def update_action_logic_cb(req: rpc.project.UpdateActionLogicRequest) -> \
-        Union[rpc.project.UpdateActionLogicResponse, hlp.RPC_RETURN_TYPES]:
+async def update_action_logic_cb(req: rpc.project.UpdateActionLogicRequest, ui: WsClient) -> None:
 
     assert glob.PROJECT
     assert glob.SCENE
@@ -733,11 +805,11 @@ async def update_action_logic_cb(req: rpc.project.UpdateActionLogicRequest) -> \
 
     for inp in req.args.inputs:
         if inp.default not in allowed_values:
-            return False, "Unknown input value."
+            raise Arcor2Exception("Unknown input value.")
 
     for out in req.args.outputs:
         if out.default not in allowed_values:
-            return False, "Unknown output value."
+            raise Arcor2Exception("Unknown output value.")
 
     action.inputs = req.args.inputs
     action.outputs = req.args.outputs
@@ -748,8 +820,7 @@ async def update_action_logic_cb(req: rpc.project.UpdateActionLogicRequest) -> \
 
 
 @no_project
-async def delete_project_cb(req: rpc.project.DeleteProjectRequest) -> \
-        Union[rpc.project.DeleteProjectResponse, hlp.RPC_RETURN_TYPES]:
+async def delete_project_cb(req: rpc.project.DeleteProjectRequest, ui: WsClient) -> None:
 
     project = await storage.get_project(req.args.id)
     await storage.delete_project(req.args.id)
@@ -758,11 +829,13 @@ async def delete_project_cb(req: rpc.project.DeleteProjectRequest) -> \
     return None
 
 
-async def rename_project_cb(req: rpc.project.RenameProjectRequest) -> \
-        Union[rpc.project.RenameProjectResponse, hlp.RPC_RETURN_TYPES]:
+async def rename_project_cb(req: rpc.project.RenameProjectRequest, ui: WsClient) -> None:
 
     unique_name(req.args.new_name, (await project_names()))
 
+    if req.dry_run:
+        return None
+
     async with managed_project(req.args.project_id) as project:
 
         project.name = req.args.new_name
@@ -773,11 +846,13 @@ async def rename_project_cb(req: rpc.project.RenameProjectRequest) -> \
     return None
 
 
-async def copy_project_cb(req: rpc.project.CopyProjectRequest) -> \
-        Union[rpc.project.CopyProjectResponse, hlp.RPC_RETURN_TYPES]:
+async def copy_project_cb(req: rpc.project.CopyProjectRequest, ui: WsClient) -> None:
 
     unique_name(req.args.target_name, (await project_names()))
 
+    if req.dry_run:
+        return None
+
     async with managed_project(req.args.source_id, make_copy=True) as project:
 
         project.name = req.args.target_name
@@ -787,8 +862,7 @@ async def copy_project_cb(req: rpc.project.CopyProjectRequest) -> \
     return None
 
 
-async def update_project_description_cb(req: rpc.project.UpdateProjectDescriptionRequest) -> \
-        Union[rpc.project.UpdateProjectDescriptionResponse, hlp.RPC_RETURN_TYPES]:
+async def update_project_description_cb(req: rpc.project.UpdateProjectDescriptionRequest, ui: WsClient) -> None:
 
     async with managed_project(req.args.project_id) as project:
 
@@ -800,8 +874,7 @@ async def update_project_description_cb(req: rpc.project.UpdateProjectDescriptio
     return None
 
 
-async def update_project_has_logic_cb(req: rpc.project.UpdateProjectHasLogicRequest) -> \
-        Union[rpc.project.UpdateProjectHasLogicResponse, hlp.RPC_RETURN_TYPES]:
+async def update_project_has_logic_cb(req: rpc.project.UpdateProjectHasLogicRequest, ui: WsClient) -> None:
 
     async with managed_project(req.args.project_id) as project:
 
@@ -823,13 +896,16 @@ async def update_project_has_logic_cb(req: rpc.project.UpdateProjectHasLogicRequ
 
 
 @project_needed
-async def rename_action_point_joints_cb(req: rpc.project.RenameActionPointJointsRequest) -> \
-        Union[rpc.project.RenameActionPointJointsResponse, hlp.RPC_RETURN_TYPES]:
+async def rename_action_point_joints_cb(req: rpc.project.RenameActionPointJointsRequest, ui: WsClient) -> None:
 
     assert glob.PROJECT
 
     ap, joints = glob.PROJECT.ap_and_joints(req.args.joints_id)
     unique_name(req.args.new_name, ap.joints_names())
+
+    if req.dry_run:
+        return None
+
     joints.name = req.args.new_name
     glob.PROJECT.update_modified()
     asyncio.ensure_future(notif.broadcast_event(events.JointsChanged(events.EventType.UPDATE_BASE, data=joints)))
@@ -838,13 +914,17 @@ async def rename_action_point_joints_cb(req: rpc.project.RenameActionPointJoints
 
 
 @project_needed
-async def rename_action_point_orientation_cb(req: rpc.project.RenameActionPointOrientationRequest) -> \
-        Union[rpc.project.RenameActionPointOrientationResponse, hlp.RPC_RETURN_TYPES]:
+async def rename_action_point_orientation_cb(req: rpc.project.RenameActionPointOrientationRequest, ui: WsClient) -> \
+        None:
 
     assert glob.PROJECT
 
     ap, ori = glob.PROJECT.ap_and_orientation(req.args.orientation_id)
     unique_name(req.args.new_name, ap.orientation_names())
+
+    if req.dry_run:
+        return None
+
     ori.name = req.args.new_name
     glob.PROJECT.update_modified()
     asyncio.ensure_future(notif.broadcast_event(events.OrientationChanged(events.EventType.UPDATE_BASE, data=ori)))
@@ -853,12 +933,15 @@ async def rename_action_point_orientation_cb(req: rpc.project.RenameActionPointO
 
 
 @project_needed
-async def rename_action_cb(req: rpc.project.RenameActionRequest) -> \
-        Union[rpc.project.RenameActionResponse, hlp.RPC_RETURN_TYPES]:
+async def rename_action_cb(req: rpc.project.RenameActionRequest, ui: WsClient) -> None:
 
     assert glob.PROJECT
 
     unique_name(req.args.new_name, glob.PROJECT.action_user_names())
+
+    if req.dry_run:
+        return None
+
     act = glob.PROJECT.action(req.args.action_id)
     act.name = req.args.new_name
 
