@@ -1,6 +1,6 @@
 import abc
-from typing import Set
-from arcor2 import DynamicParamDict
+from typing import FrozenSet
+from arcor2 import DynamicParamDict, CancelDict
 from arcor2.docstring import parse_docstring
 
 
@@ -10,6 +10,7 @@ class Service(metaclass=abc.ABCMeta):
     """
 
     DYNAMIC_PARAMS: DynamicParamDict = {}
+    CANCEL_MAPPING: CancelDict = {}
 
     def __init__(self, configuration_id: str) -> None:
         self.configuration_id = configuration_id
@@ -23,5 +24,5 @@ class Service(metaclass=abc.ABCMeta):
         return parse_docstring(cls.__doc__)["short_description"]
 
     @staticmethod
-    def get_configuration_ids() -> Set[str]:
-        return set()
+    def get_configuration_ids() -> FrozenSet[str]:
+        return frozenset()
