@@ -1,4 +1,4 @@
-from typing import Set, Optional
+from typing import FrozenSet, Optional
 import time
 
 from arcor2.services import Service
@@ -16,8 +16,8 @@ class TimeService(Service):
         self._rate_start_time: Optional[float] = None
 
     @staticmethod
-    def get_configuration_ids() -> Set[str]:
-        return {"default"}
+    def get_configuration_ids() -> FrozenSet[str]:
+        return frozenset({"default"})
 
     @action
     def sleep(self, seconds: float = 1.0) -> None:
@@ -47,5 +47,14 @@ class TimeService(Service):
         if dif > 0:
             time.sleep(dif)
 
+    @action
+    def time_ns(self) -> int:
+        """
+        Returns nanoseconds since Unix Epoch.
+        :return:
+        """
+        return time.time_ns()
+
     sleep.__action__ = ActionMetadata(blocking=True)  # type: ignore
     rate.__action__ = ActionMetadata(blocking=True)  # type: ignore
+    time_ns.__action__ = ActionMetadata(blocking=True)  # type: ignore
