@@ -442,43 +442,42 @@ def global_action_points_class(project: Project) -> str:
                 type_comment=None)],
         decorator_list=[])
 
-    for obj in project.objects:
-        for ap in obj.action_points:
-            fd = FunctionDef(
-                name=clean(ap.id),  # TODO avoid possible collisions
-                args=arguments(
-                    args=[arg(arg='self', annotation=None, type_comment=None)],
-                    vararg=None,
-                    kwonlyargs=[],
-                    kw_defaults=[],
-                    kwarg=None,
-                    defaults=[]),
-                body=[Return(value=Call(
-                    func=Attribute(
+    for ap in project.action_points:
+        fd = FunctionDef(
+            name=clean(ap.name),  # TODO avoid possible collisions
+            args=arguments(
+                args=[arg(arg='self', annotation=None, type_comment=None)],
+                vararg=None,
+                kwonlyargs=[],
+                kw_defaults=[],
+                kwarg=None,
+                defaults=[]),
+            body=[Return(value=Call(
+                func=Attribute(
+                    value=Attribute(
                         value=Attribute(
-                            value=Attribute(
-                                value=Name(
-                                    id='self',
-                                    ctx=Load()),
-                                attr='_res',
+                            value=Name(
+                                id='self',
                                 ctx=Load()),
-                            attr='project',
+                            attr='_res',
                             ctx=Load()),
-                        attr='action_point',
+                        attr='project',
                         ctx=Load()),
-                    args=[Str(
-                        s=ap.id,
-                        kind='')],
-                    keywords=[]))],
-                decorator_list=[Name(
-                    id='property',
-                    ctx=Load())],
-                returns=Name(
-                    id='ActionPoint',
+                    attr='action_point',
                     ctx=Load()),
-                type_comment=None)
+                args=[Str(
+                    s=ap.id,
+                    kind='')],
+                keywords=[]))],
+            decorator_list=[Name(
+                id='property',
+                ctx=Load())],
+            returns=Name(
+                id='ActionPoint',
+                ctx=Load()),
+            type_comment=None)
 
-            cls_def.body.append(fd)
+        cls_def.body.append(fd)
 
     tree.body.append(cls_def)
     return tree_to_str(tree)
@@ -533,56 +532,55 @@ def global_actions_class(project: Project) -> str:
                 type_comment=None)],
         decorator_list=[])
 
-    for obj in project.objects:
-        for ap in obj.action_points:
-            for action in ap.actions:
-                ac_obj, ac_type = action.parse_type()
+    for ap in project.action_points:
+        for action in ap.actions:
+            ac_obj, ac_type = action.parse_type()
 
-                m = FunctionDef(
-                    name=action.id,
-                    args=arguments(
-                        args=[arg(
-                            arg='self',
-                            annotation=None,
-                            type_comment=None)],
-                        vararg=None,
-                        kwonlyargs=[],
-                        kw_defaults=[],
-                        kwarg=None,
-                        defaults=[]),
-                    body=[Expr(value=Call(
-                        func=Attribute(
-                            value=Subscript(
-                                value=Attribute(
-                                    value=Attribute(
-                                        value=Name(
-                                            id='self',
-                                            ctx=Load()),
-                                        attr='_res',
-                                        ctx=Load()),
-                                    attr='all_instances',
-                                    ctx=Load()),
-                                slice=Index(value=Str(
-                                    s=ac_obj,
-                                    kind='')),
-                                ctx=Load()),
-                            attr=ac_type,
-                            ctx=Load()),
-                        args=[Attribute(
+            m = FunctionDef(
+                name=clean(action.name),
+                args=arguments(
+                    args=[arg(
+                        arg='self',
+                        annotation=None,
+                        type_comment=None)],
+                    vararg=None,
+                    kwonlyargs=[],
+                    kw_defaults=[],
+                    kwarg=None,
+                    defaults=[]),
+                body=[Expr(value=Call(
+                    func=Attribute(
+                        value=Subscript(
                             value=Attribute(
-                                value=Name(
-                                    id='self',
+                                value=Attribute(
+                                    value=Name(
+                                        id='self',
+                                        ctx=Load()),
+                                    attr='_res',
                                     ctx=Load()),
-                                attr='_res',
+                                attr='all_instances',
                                 ctx=Load()),
-                            attr=action.id,
-                            ctx=Load())],
-                        keywords=[]))],
-                    decorator_list=[],
-                    returns=None,
-                    type_comment=None)
+                            slice=Index(value=Str(
+                                s=ac_obj,
+                                kind='')),
+                            ctx=Load()),
+                        attr=ac_type,
+                        ctx=Load()),
+                    args=[Attribute(
+                        value=Attribute(
+                            value=Name(
+                                id='self',
+                                ctx=Load()),
+                            attr='_res',
+                            ctx=Load()),
+                        attr=clean(action.name),
+                        ctx=Load())],
+                    keywords=[]))],
+                decorator_list=[],
+                returns=None,
+                type_comment=None)
 
-                cls_def.body.append(m)
+            cls_def.body.append(m)
 
     tree.body.append(cls_def)
     return tree_to_str(tree)
@@ -590,14 +588,14 @@ def global_actions_class(project: Project) -> str:
 
 def derived_resources_class(project: Project) -> str:
     # TODO temporary and ugly solution of circular import
+    import arcor2.resources
     from arcor2.resources import ResourcesBase
 
     tree = Module(body=[])
 
-    parameters = [act.id for obj in project.objects for aps in obj.action_points for act in aps.actions]
+    parameters = [(act.id, clean(act.name)) for aps in project.action_points for act in aps.actions]
 
-    # TODO avoid having "arcor2.resources" as string - how?
-    add_import(tree, "arcor2.resources", ResourcesBase.__name__)
+    add_import(tree, arcor2.resources.__name__, ResourcesBase.__name__)
 
     derived_cls_name = "Resources"
 
@@ -608,11 +606,11 @@ def derived_resources_class(project: Project) -> str:
                        attr='__init__', ctx=Load()), args=[Str(s=project.id)],
         keywords=[]))]
 
-    for param in parameters:
-        init_body.append(Assign(targets=[get_name_attr("self", "_" + param, Store)],
+    for a_id, a_name in parameters:
+        init_body.append(Assign(targets=[get_name_attr("self", "_" + a_name, Store)],
                                 value=Call(
                                     func=get_name_attr("self", "parameters"),
-                                    args=[Str(s=param)], keywords=[])))
+                                    args=[Str(s=a_id)], keywords=[])))
 
     cls_def = ClassDef(name=derived_cls_name,
                        bases=[Name(id=ResourcesBase.__name__, ctx=Load())],
@@ -625,9 +623,9 @@ def derived_resources_class(project: Project) -> str:
 
     tree.body.append(cls_def)
 
-    for param in parameters:
+    for a_id, a_name in parameters:
         cls_def.body.append(FunctionDef(
-            name=param,
+            name=a_name,
             args=arguments(
                 args=[arg(
                     arg='self',
@@ -643,11 +641,11 @@ def derived_resources_class(project: Project) -> str:
                     func=Attribute(value=Name(id='self', ctx=Load()), attr='print_info', ctx=Load()),
                     args=[
                         Str(
-                            s=param,
+                            s=a_id,
                             kind=''),
-                        get_name_attr('self', '_' + param)],
+                        get_name_attr('self', '_' + a_name)],
                     keywords=[])),
-                Return(value=get_name_attr('self', '_' + param))],
+                Return(value=get_name_attr('self', '_' + a_name))],
             decorator_list=[Name(
                 id='property',
                 ctx=Load())],
